#==============================================================================
#==============================================================================
#
#                             myCOMBINAT.PUB.R
#
#  set of functions for combinatorial analysis of community diversity effects
#
#                         Benoît JAILLARD, January 2017
#                        
#                            benoit.jaillard@inra.fr
#
#==============================================================================
#==============================================================================



#==============================================================================
#  Return a vector continuously indexed from 1 to max(index)
#==============================================================================
compact.index <- function(v) {

  res <- v
  set <- sort(unique(v))
  for (i in seq_along(set)) res[v == set[i]] <- i

  return(res)
}



#==============================================================================
#  Cluster the assemblages by assembly motifs
# 
#  Inputs  : affectElt and mat Occurrence
#  Outputs : the vector of motifs to which belong the assemblages
#==============================================================================
affect.motifs <- function(affectElt, mOccur) {
  
  EltClasses <- t( t(mOccur) * compact.index(affectElt) )
  
  myfct1  <- function(x) { x[x != 0] }
  tmp     <- apply(EltClasses[ , , drop = FALSE], MARGIN = 1, FUN = myfct1)

  myfct2  <- function(x) { sort(unique(x)) }
  tmp     <- lapply(tmp, FUN = myfct2)
  
  size    <- unlist(lapply(tmp, FUN = length))
  setSize <- sort(unique(size))
  
  motif     <- 1
  AssMotifs <- integer(dim(mOccur)[1])
  for (siz in seq_along(setSize)) {
    
    index <- which(size == setSize[siz])
    
    while (length(index) > 0) {
      ind <- 1
      ref <- tmp[[index[ind]]]
      for (i in 2:length(index))
        if (identical(ref, tmp[[index[i]]])) ind <- c(ind, i)
      
      AssMotifs[index[ind]] <- motif
      
      motif <- motif + 1
      index <- index[-ind]
    }
  }
  
  return(AssMotifs)
}



#==============================================================================
#  Cluster the assemblages by assembly motifs
# 
#  Inputs  : a tree and mat Occurrence
#  Outputs : the matrix of motifs to which belong the assemblages
#==============================================================================
mAffect.motifs <- function(tree, mOccur) {

  nbclTot <- length(tree$cor)
  nbclMax <- which(tree$cor == max(tree$cor))[1]
  
  mAssMotifs <- matrix(0, nrow = nbclTot, ncol = dim(mOccur)[1])
  for (nbcl in seq_len(nbclMax))
    mAssMotifs[nbcl, ] <- affect.motifs(cut.tree(tree, nbcl), mOccur)

  if (nbclMax < nbclTot) 
    for (nbcl in (nbclMax + 1):nbclTot)
      mAssMotifs[nbcl, ] <- mAssMotifs[nbclMax, ]
  
  colnames(mAssMotifs) <- c(1:dim(mOccur)[1])
  rownames(mAssMotifs) <- c(1:nbclTot)

  return(mAssMotifs)
}



#==============================================================================
#  Return the names (in minus letters) of elements, clusters of elements,
#              and all possible assembly motifs associated to element clusters
#
#  Input:  mOccur    : matrix of occurrence
#          affectElt : affectation of species into classes
#  Output: names.elements         : names of elements
#          names.element.clusters : names of clusters of elements
#          names.assembly.motifs  : names of elemental assembly motifs
#==============================================================================
names.assembly <- function(affectElt, mOccur) {
  
  # names of element clusters 
  affectlt  <- compact.index(affectElt)
  nomAffect <- myLetters[affectElt]
  
  # names of assembly motifs 
  EltClasses <- t( t(mOccur) * affectElt )

  myfct1   <- function(x) { x[x != 0] }
  tmp      <- apply(EltClasses[ , , drop = FALSE], MARGIN = 1, FUN = myfct1)
  
  myfct2   <- function(x) { sort(unique(x)) }
  tmp      <- lapply(tmp, FUN = myfct2)
  
  nomMotif <- character(length(tmp))
  for (i in seq_along(tmp))  
    for (j in seq_along(tmp[[i]]))
      nomMotif[i] <- paste(nomMotif[i], myLetters[tmp[[i]][j]], sep = "")

  noms        <- list(colnames(mOccur), nomAffect, nomMotif)
  names(noms) <- c("elements", "clusters", "motifs")
  
  return(noms)
}


#system.time(for (i in 1:100000) affect_motif.name(x)    )

#==============================================================================
#  Plot the contents of clusters of elements or motifs
#==============================================================================
plot.legend <- function(noms) {
  
  nbcol <- 1
  nbrow <- which(max(noms$clusters) == myLetters)
  for (i in seq_len(nbrow)) {
    a     <- noms$elements[which(noms$clusters == myLetters[i])]
    nbcol <- max(nbcol, length(a))
  }
  
  tmp <- matrix("", nrow = nbrow, ncol = nbcol)
  rownames(tmp) <- myLetters[1:nbrow]
  colnames(tmp) <- c(1:nbcol)
  
  for (i in seq_len(nbrow)) {
    a <- noms$elements[which(noms$clusters == myLetters[i])]
    for (j in seq_along(a)) tmp[i, j] <- a[j]
  }
  
  thres <- 80
  tab   <- NULL
  for (i in seq_len(dim(tmp)[1])) {
    j <- 1
    a <- paste(letters[i %% length(letters)], "= {", tmp[i, j])
    while (j < dim(tmp)[2]) {
      
      while ((nchar(a) < thres) & (j < dim(tmp)[2])) {
        j <- j + 1
        if (tmp[i, j] != "") a <- paste(a, tmp[i, j], sep = ", ")
      }  
      a <- paste(a, ",", sep = "")  
      
      tab <- c(tab, a)
      a <- ""
      for (k in 1:7) a <- paste(a, " ", sep = "")
    }
    tab[length(tab)] <- paste(tab[length(tab)], "}", sep = " ")
  } 
  
  tab  <- c("", "Legend:", tab, "")
  
  plot(x = seq_along(tab), y = seq_along(tab),
       xlab = "",          ylab = "",
       type = "n", axes = FALSE)
  axis(side = 4, at = seq_along(tab), rev(tab),
       pos = 1, tick = FALSE, las = 2, col = "black")
}



#==============================================================================
#==============================================================================
#
#                    END of FILE myCOMBINAT.PUB.R
#
#==============================================================================
#==============================================================================



