#==============================================================================
#==============================================================================
#
#                               mySEPARATING.PUB.R
#                               
#                      set of functions for plotting
#             the interaction effects and composition effects
#             
#                           Benoît JAILLARD, January 2017
#       
#                            benoit.jaillard@inra.fr
#
#==============================================================================
#==============================================================================



#==============================================================================
#  Plot a vector of points with or without std bars
#
#  Inputs : x, y : vectors of coordinates
#           figure, couleur, nom : informations to plot the vectors
#           opt.std : with (TRUE) or without( FALSE) error-bars
#           scale.log : plot in log-scale (TRUE) or not (FALSE).
#           If TRUE, the vectors x and y must be input as log2(x) and log2(y)
#
#==============================================================================
points.sd <- function(x, y,
                      figure, couleur, nom,
                      opt.std = TRUE,
                      scale.log = FALSE) {

  mx <- mean(x, na.rm = TRUE)
  my <- mean(y, na.rm = TRUE)

  #----------------------------------------------------------------------------
  # plot at first the error-bars
  #----------------------------------------------------------------------------
  if ((opt.std == TRUE) & (length(x) > 1)) {
    dx  <- sd(x)
    dy  <- sd(y)

    if (scale.log == FALSE) {
      if (dx != 0) arrows(x0 = mx - dx, y0 = my,
                          x1 = mx + dx, y1 = my,
                          col = couleur, length = 0.1, angle = 90, code = 3,
                          lwd = 1, lty = "solid")
      if (dy != 0) arrows(x0 = mx, y0 = my - dy,
                          x1 = mx, y1 = my + dy,
                          col = couleur, length = 0.1, angle = 90, code = 3,
                          lwd = 1, lty = "solid")
    } else {

      if ((dx != 0) & ((mx - dx) > 0))
        arrows(x0 = log2(mx - dx), y0 = log2(my),
               x1 = log2(mx + dx), y1 = log2(my),
               col = couleur, length = 0.1, angle = 90, code = 3,
               lwd = 1, lty = "solid")
      if ((dy != 0) & ((my - dy) > 0))
        arrows(x0 = log2(mx), y0 = log2(my - dy),
               x1 = log2(mx), y1 = log2(my + dy),
               col = couleur, length = 0.1, angle = 90, code = 3,
               lwd = 1, lty = "solid")
    }
  }

  #----------------------------------------------------------------------------
  # plot second the mean points with a white background
  #----------------------------------------------------------------------------
  if (scale.log == FALSE) {
    points(x = mx, y = my,
           type = "p", pch = figure, col = couleur, bg = "white", cex = 4)
    text(x = mx, y = my,
         labels = nom, pos = 4, col = couleur, bg = "white")

  } else {

    points(x = log2(mx), y = log2(my),
           type = "p", pch = figure, col = couleur, bg = "white", cex = 4)
    text(x = log2(mx), y = log2(my),
         labels = nom, pos = 4, col = couleur, bg = "white")
  }

}



#==============================================================================
#  Plot alpha x beta
#
#  Inputs : x, y : alpha, beta vectors of coordinates
#           figure, couleur, nom : informations to plot the vectors
#           opt.std : with (TRUE) or without( FALSE) error-bars
#           scale.log : plot in log-scale (TRUE) or not (FALSE).
#           If TRUE, the vectors x and y must be input as x and y
#
#==============================================================================
plot.alphaVSbeta <- function(x, y, xlab, ylab,
                             xlim = range(x), ylim = range(y),
                             titre = "",
                             scale.log = FALSE) {

  R2 <- cor2(x, y)

  if (scale.log == FALSE) {

    plot(x = x, xlim = xlim, xlab = xlab,
         y = y, ylim = ylim, ylab = ylab,
         main = titre,
         las = 1, type = "n", tck = 0.02)

    abline(h = 1, col = "black")
    abline(v = mean(x), h = mean(y), col = "blue")

    points(x = x, y = y, cex = 2, pch = 21, bg = "white")

    text(x = min(x) + 3/4*(max(x) - min(x)), y = min(y),
         labels = paste("R2 = ", signif(R2, digits = 3), sep = ""),
         pos = 3, col = "blue")

  } else {

    log2x <- log2(x)
    log2y <- log2(y)

    plot(x = log2x, xlim = log2(xlim),
         xlab = paste("log2(", xlab, ")", sep = ""),
         y = log2y, ylim = log2(ylim),
         ylab = paste("log2(", ylab, ")", sep = ""),
         main = paste("log2(", titre, ")", sep = ""),
         las = 1, type = "n", tck = 0.02)

    abline(h = 0, v = 0, col = "black")
    abline(v = log2(mean(x)), h = log2(mean(y)), col = "blue")

    points(x = log2x, y = log2y, cex = 2, pch = 21, bg = "white")

    text(x = min(log2x) + 3/4*(max(log2x) - min(log2x)), y = min(log2y),
         labels = paste("R2 = ", signif(R2, digits = 3), sep = ""),
         pos = 3, col = "blue")
  }

}



#==============================================================================
#  Remove one or more species from data
#     and return the index.row and index.col to select
#
#     Inputs : mOccur and Fobs : matrix of occurrence and function
#             elements : a vector of names of elements to remove.
#                        Names of elements must be in colnames of mOccur.
#     Outputs : mOccur and Fobs, without the elements to remove
#
#==============================================================================
rm.elements <- function(mOccur, fct, elements) {

  index.row <- c(1:dim(mOccur)[1])
  index.col <- c(1:dim(mOccur)[2])

  for (elt in seq_along(elements)) {
    index.row <- setdiff(index.row, which(mOccur[ ,elements[elt]] == 1))
    index.col <- setdiff(index.col, which(elements[elt] == colnames(mOccur)))
  }

  mOccur <- mOccur[index.row, index.col]
  fct    <- fct[index.row]

  res        <- list(mOccur, fct)
  names(res) <- c("mOccur", "fct")

  return(res)
}



#==============================================================================
#  Decompose the observed performance into a.inter and a.comp ratios
#==============================================================================
multiplicative.decomposition <- function(mOccur, Fobs,
                                         rm.mono  = TRUE) {

  #----------------------------------------------------------------------------
  # Check if all monocultures are observed
  #----------------------------------------------------------------------------
  size <- apply(mOccur, MARGIN = 1, FUN = sum)
  mono <- which(size == 1)
  if (length(mono) == 0) stop("There is no monoculture in mOccur")

  elements <- which(apply(mOccur[mono,], MARGIN = 2, FUN = sum) != 0)

  if (length(elements) < dim(mOccur)[2]) {
    missing <- setdiff(colnames(mOccur), colnames(mOccur)[elements])
    res     <- remove.elements(mOccur, Fobs, missing)

    mOccur  <- res$mOccur
    Fobs    <- res$fct
    size    <- apply(mOccur, MARGIN = 1, FUN = sum)
    mono    <- which(size == 1)
  }

  #----------------------------------------------------------------------------
  # Look for and compute the monoculture performance
  #----------------------------------------------------------------------------
  Fmono <- apply(mOccur[mono,] * Fobs[mono], MARGIN = 2, FUN = sum) /
           apply(mOccur[mono,], MARGIN = 2, FUN = sum)
  
  Fref   <- (mOccur %*% Fmono) / size
  Fscale <- mean(Fmono)


  #----------------------------------------------------------------------------
  # Compute the interaction and composition effects
  #----------------------------------------------------------------------------
  alpha     <- as.vector(Fobs/Fref)
  beta      <- as.vector(Fref/Fscale)

  #----------------------------------------------------------------------------
  # Remove the monocultures
  #----------------------------------------------------------------------------
  if (rm.mono == TRUE) {
    index   <- which(size != 1)
    size    <- size[index]
    mOccur  <- mOccur[index, ]
    Fobs    <- Fobs[index]
    alpha   <- alpha[index]
    beta    <- beta[index]
  }


  res        <- list(size, mOccur,
              Fobs, alpha, beta, Fmono, Fscale)
  names(res) <- c("size", "mOccur",
                  "Fobs", "alpha", "beta", "Fmono", "Fscale")

  return(res)
}



#==============================================================================
# Plot the histogram of fct-values
#==============================================================================
ghisto <- function(x, namex, scale.log = FALSE) {

  if (scale.log == FALSE) {
    hist(x, xlab = namex, ylab = "Density", prob = TRUE, main = namex,
         las = 1, breaks = 20)
    abline(v = 1, col = "black")
    abline(v = gmean(x), col = "blue")

  } else {
    titre <- paste("log2(", namex, ")", sep = "")
    hist(log2(x), xlab = titre, ylab = "Density", prob = TRUE, main = titre,
         las = 1, breaks = 20)
    abline(v = 0, col = "black")
    abline(v = log2(gmean(x)), col = "blue")
  }

}



#==============================================================================
#==============================================================================
#
#                         END of FILE mySEPARATING.R
#
#==============================================================================
#==============================================================================
