#==============================================================================
#==============================================================================
#
#                               myTOOLS.PUB.R
#                               
#                           Benoît JAILLARD, January 2017
#       
#                            benoit.jaillard@inra.fr
#
#==============================================================================
#==============================================================================


library("agricolae")



#==============================================================================
#  defining usefull constantes
#==============================================================================

figures  <- c(21, 22, 24, 23, 25)
# 21 = "circle", 22 = "square", 23 = "diamond",
# 24 = triangle point-up", 25 = "triangle point-down"

couleurs <- c("red3", "blue3", "orange2", "turquoise3", "magenta", "green4")

myLetters <- c(letters, LETTERS)




#===============================================================================
#===============================================================================
#
#   Functions and Procedures
#
#===============================================================================
#===============================================================================

#===============================================================================
#  Function : to generate vectors of colours and symbols
#                             as many long as the number of studied communities
#  Input: base symbol vector, and a vector of elements to plot
#  Output: a vector of symbols of length > length(vector)-
#===============================================================================
check.symbol <- function(symbol, long) {
  
  res <- NULL
  while (length(res) < long) res <- c(res, symbol)
  return(res)
}



#------------------------------------------------------------------------------
#  Return the arithmetic mean value
#------------------------------------------------------------------------------
amean <- function(x, na.rm = TRUE) {
  
  if (na.rm == TRUE) x <- x[!is.na(x)]
  ifelse(length(x), sum(x) / length(x), NA)
}



#------------------------------------------------------------------------------
#  Return the geometric standard deviation
#------------------------------------------------------------------------------
asd <- function(x, na.rm = TRUE) {
  
  if (na.rm == TRUE) x <- x[!is.na(x)]
  
  mu <- amean(x)
  ifelse(length(x), sqrt( sum((x - mu) ^ 2) / length(x) ), NA)
}



#==============================================================================
#  Return the geometric mean and std
#==============================================================================

#------------------------------------------------------------------------------
#  Return the geometric mean value
#------------------------------------------------------------------------------
gmean <- function(x, na.rm = TRUE) {
  
  if (na.rm == TRUE) x <- x[!is.na(x)]
  ifelse(length(x), prod(x) ^ (1/length(x)), NA)
}



#------------------------------------------------------------------------------
#  Return the geometric standard deviation
#------------------------------------------------------------------------------
gsd <- function(x, na.rm = TRUE) {
  
  if (na.rm == TRUE) x <- x[!is.na(x)]
  ifelse(length(x),
         exp( sqrt( sum( log(x/gmean(x)) ^ 2 ) / length(x) ) ),
         NA)
}



#------------------------------------------------------------------------------
#  Return the Pearson' R2 of a linear regression
#------------------------------------------------------------------------------
cor2 <- function(x, y, na.rm = TRUE) {
  
  if (na.rm == TRUE) {
    index <- !(is.na(x) | is.na(y))
    x     <- x[index]
    y     <- y[index]
  }
  
  res <- NA
  if (length(x) > 0)
  {
    z1 <- var(x)
    z2 <- var(y)
    z3 <- cov(x,y)
    
    if ((z1 > 0) && (z2 > 0)) res <- (z3 ^ 2) / (z1 * z2)
  }
  
  return(res)
}



#------------------------------------------------------------------------------
#  Return the Coefficient of Determination R2 of Predicted values
#------------------------------------------------------------------------------
R2mse <- function(pred, obs, na.rm = TRUE) {
  
  if (na.rm == TRUE) {
    index <- !(is.na(pred) | is.na(obs))
    pred  <- pred[index]
    obs   <- obs[index]
  }
  
  res <- NA
  if (length(obs) > 1) {
    tss   <- sum((obs - amean(obs)) ^ 2)
    rss   <- sum((obs - pred) ^ 2)
    res   <- (tss - rss) / tss
  }
  
  return(res)
}



#------------------------------------------------------------------------------
#  Return the Probability associated to a R2mse 
#     ( with "r" for "Residual", "e" for "Explained" and "t" for "Total")
#------------------------------------------------------------------------------
pmse <- function(prd, obs, nbclasses, na.rm = TRUE)
{
  if (nbclasses == 1) stop("nbclasses should be higher than one")
  
  if (na.rm == TRUE) {
    index <- !(is.na(prd) | is.na(obs))
    prd   <- prd[index]
    obs   <- obs[index]
  }
  
  res <- 1
  if (length(obs) > 1) {
    tss   <- sum((obs - amean(obs)) ^ 2)
    rss   <- sum((obs - prd) ^ 2)
    ess   <- tss - rss
    
    nbfdt <- length(obs)
    nbfde <- nbclasses - 1
    nbfdr <- nbfdt - nbfde
    
    if ( (nbfdr > 0) && (nbfde > 0) ) {
      Fratio <- (ess / nbfde) / (rss / nbfdr)
      res    <- 1 - pf(Fratio, nbfde, nbfdr)
    }
  }
  
  return(res)
}



#==============================================================================
# Variance analysis with HSD posthoc test
#
#  Inputs : x        = vector of data,
#           clusters = vector of factors
#           pvalue   = value of 1er risk of first species
#  Output : results
#==============================================================================
test.posthoc <- function(x, clusters, pvalue = 0.05) {
  
  # check the inputs
  if (!is.factor(clusters)) clusters <- as.factor(clusters)
  
  if (sum(is.na(x)) > 0) {
    x        <- x[!is.na(x)]
    clusters <- clusters[!is.na(x)]
  }
  
  levels  <- sort(unique(clusters))
  nblevel <- length(levels)
  
  res   <- NA
  index <- which(table(clusters) > 1)
  if (length(index) > 1) {
    
    # analyse the variances
    model <- aov(x ~ clusters)
    res1  <- TukeyHSD(x = model, "clusters", conf.level = (1 - pvalue))
    res2  <- agricolae::HSD.test(y = model, "clusters",
                                 alpha = pvalue, group = TRUE)
    
    # build the matrix of pValues
    mat <- mletters <- matrix(0, nrow = nblevel, ncol = nblevel)
    colnames(mat) <- rownames(mat) <- levels
    count <- 0
    for (iter1 in 1:(nblevel - 1))
      for (iter2 in (iter1 + 1):nblevel) {
        count <- count + 1
        mat[iter2, iter1] <- res1$clusters[count,"p adj"]
      }
    
    mat <- mat + t(mat)
    diag(mat) <- 1
    
    # outputs
    res <- res2
    res$matrix <- mat
    names(res) <- c("statistics", "parameters", "means",
                    "comparison", "groups", "matrix")
  }
  
  return(res)
}





#==============================================================================
#==============================================================================
#
#                         END of FILE mySEPARATING.R
#
#==============================================================================
#==============================================================================
